/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: Action.H 403 2006-02-10 17:55:49Z eduardo $ */
/** @file Action.H
 *  @brief EPP Action Class
 */

#ifndef __ACTION_H__
#define __ACTION_H__

#include <string>

#include "libepp_nicbr.H"

#include "DomParser.H"

using std::string;

LIBEPP_NICBR_NS_BEGIN

/// EPP Action Class
class Action
{
public:
  /// Action Types
  enum Type {
    LOGIN = 0,
    LOGOUT,
    HELLO,
    POLL,
    CONTACT_CHECK = 10,
    CONTACT_CREATE,
    CONTACT_INFO,
    CONTACT_TRANSFER,
    CONTACT_UPDATE,
    DOMAIN_CHECK = 20,
    DOMAIN_CREATE,
    DOMAIN_INFO,
    DOMAIN_RENEW,
    DOMAIN_UPDATE,
    ORG_CHECK = 30,
    ORG_CREATE,
    ORG_INFO,
    ORG_UPDATE
  };

  /// Constructor that forces childs to set their types
  /**
     @param type Action type
  */
  Action(const Type &type) : _type(type) {}

  /// virtual destructor
  virtual ~Action() {}
  
  /// Sets XML template
  /**
     @param xml_template   XML command template
     @param clTRID         client transaction ID
  */
  virtual void set_xml_template(const string &xml_template, 
				const string &clTRID) = 0;

  /// Returns XML command
  /**
     @return XML command
  */
  string get_xml_command() { return _xml; }

  /// Returns Action type
  /**
     @return Action type
  */
  Type who_am_i() { return _type; }

  /// Pure virtual method to set response from a XML document
  /**
     @param xml_payload   XML document
     @param parser        reference to the XML parser
   */
  virtual void set_response(const string &xml_payload, DomParser *parser) = 0;

  /// Returns response
  /**
     @return response
   */
  virtual Response* get_response() = 0;

protected:
  /// Action type
  Type _type;

  /// XML command
  string _xml;

private:
  Action();

};

LIBEPP_NICBR_NS_END
#endif //__ACTION_H__
