/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: DomainCreate.cpp 345 2006-02-09 15:58:04Z milena $ */

#include "config.h"

#include "DomainCreate.H"
#include "StrUtil.H"

LIBEPP_NICBR_NS_BEGIN

void DomainCreate::set_xml_template(const string &xml_template, 
				    const string &clTRID)
{
  StrUtil str_util;

  map < string, string, less<string> > to_parse;

  //name
  to_parse["name"] = _command->get_name();

  //period
  DomainCreateCmd::Period period = _command->get_period();
  to_parse["period_time"] = str_util.to_string("%d", period.time);
  to_parse["period_unit"] = period.unit;

  //nameservers
  set<struct NameServer> nss = _command->get_nameservers();
  set<struct NameServer>::const_iterator it;
  set<string>::const_iterator it2;
  for (it = nss.begin(); it != nss.end(); it++) {
    to_parse["nameservers"] += "<domain:hostAttr>";
    to_parse["nameservers"] += "<domain:hostName>" + it->name +
      "</domain:hostName>";
    for (it2 = it->ips.begin(); it2 != it->ips.end(); it2++) {
      to_parse["nameservers"] += "<domain:hostAddr>" + (*it2) +
	"</domain:hostAddr>";
    }
    to_parse["nameservers"] += "</domain:hostAttr>";
  }

  //registrant
  to_parse["registrant"] = _command->get_registrant();

  //other contacts
  map< string, string, less<string> > contacts = _command->get_contacts();
  map< string, string, less<string> >::const_iterator it_contacts;
  for (it_contacts = contacts.begin(); it_contacts != contacts.end();
       it_contacts++) {
    to_parse["other_contacts"] +=
      "<domain:contact type='" + (*it_contacts).first + "'>" +
      (*it_contacts).second + "</domain:contact>";
  }

  //auth info
  AuthInfo authInfo;
  authInfo = _command->get_authInfo();
  string auth_info_str;
  auth_info_str = "<domain:authInfo>";
  
  if (authInfo.get_roid() == "") {
    auth_info_str += "<domain:pw>" + authInfo.get_pw() + "</domain:pw>";
  } else {
    auth_info_str += "<domain:pw roid='" + authInfo.get_roid() + "'>" + 
      authInfo.get_pw() + "</domain:pw>";
  }
  auth_info_str += "</domain:authInfo>";
  to_parse["auth_info"] = auth_info_str;

  //clTRID
  to_parse["clTRID"] = clTRID;

  _xml = str_util.parse(xml_template, to_parse, "$(", ")$");
}

LIBEPP_NICBR_NS_END
