/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: DomainUpdate.cpp 352 2006-02-09 16:45:19Z milena $ */

#include "config.h"

#include "DomainUpdate.H"
#include "StrUtil.H"

LIBEPP_NICBR_NS_BEGIN

void DomainUpdate::set_xml_template(const string &xml_template, 
				    const string &clTRID)
{
  StrUtil str_util;

  map < string, string, less<string> > to_parse;

  //name
  to_parse["name"] = _command->get_name();

  //nameservers of tag $(add)$
  set<struct NameServer> nameserver_add = 
    _command->get_nameserver_add();
  set<struct NameServer>::const_iterator it_add;
  set<string>::const_iterator it_str;
  if (nameserver_add.size() > 0) {
  to_parse["add"] = "<domain:add><domain:ns>";
  for (it_add = nameserver_add.begin(); it_add != nameserver_add.end(); 
	 it_add++) {
    to_parse["add"] += "<domain:hostAttr>";
    to_parse["add"] += "<domain:hostName>" + it_add->name + 
      "</domain:hostName>";
    for (it_str = it_add->ips.begin(); it_str != it_add->ips.end(); 
	 it_str++) {
      to_parse["add"] += "<domain:hostAddr>" + (*it_str) + 
	"</domain:hostAddr>";
    }
    to_parse["add"] += "</domain:hostAttr>";
  }
  to_parse["add"] += "</domain:ns>";
  
  //contacts of tag $(add)$
  map< string, string, less<string> > contact_add = 
    _command->get_contact_add();
  map< string, string, less<string> >::const_iterator it_contact_add;
  for (it_contact_add = contact_add.begin(); 
       it_contact_add != contact_add.end(); it_contact_add++) {
    to_parse["add"] += "<domain:contact type='" + 
      (*it_contact_add).first + "'>" + (*it_contact_add).second + 
      "</domain:contact>";
  }
  //status of tag $(add)$
  set<struct DomainUpdateCmd::Status> status_add = _command->get_status_add();
  set<struct DomainUpdateCmd::Status>::const_iterator it_add_st;
  for (it_add_st = status_add.begin(); it_add_st != status_add.end();
       it_add_st++) {
    to_parse["add"] += "<domain:status s='" + (*it_add_st).s + "' lang='" + 
      (*it_add_st).lang + "'>" + (*it_add_st).msg + "</domain:status>";
  }
    to_parse["add"] += "</domain:add>";
  }
  else
    to_parse["add"] = "";
  
  //nameservers of tag $(rem)$
  set<struct NameServer> nameserver_rem = _command->get_nameserver_rem();
  set<struct NameServer>::const_iterator it_rem;
  if (nameserver_rem.size() > 0) {
    to_parse["rem"] = "<domain:rem><domain:ns>";
    for (it_rem = nameserver_rem.begin(); it_rem != nameserver_rem.end(); 
	 it_rem++) {
      to_parse["rem"] += "<domain:hostAttr><domain:hostName>" + 
	(*it_rem).name + "</domain:hostName></domain:hostAttr>";
    }
    to_parse["rem"] += "</domain:ns>";

    //contacts of tag $(rem)$
    map< string, string, less<string> > contact_rem = 
      _command->get_contact_rem();
    map< string, string, less<string> >::const_iterator it_contact_rem;
    for (it_contact_rem = contact_rem.begin();
	 it_contact_rem != contact_rem.end(); it_contact_rem++) {
      to_parse["rem"] += "<domain:contact type='" + 
	(*it_contact_rem).first + "'>" + (*it_contact_rem).second + 
	"</domain:contact>";
    }
    //status of tag $(rem)$
    set<struct DomainUpdateCmd::Status> status_rem = _command->get_status_rem();
    set<struct DomainUpdateCmd::Status>::const_iterator it_rm_st;
    for (it_rm_st = status_rem.begin(); it_rm_st != status_rem.end();
	 it_rm_st++) {
      to_parse["rem"] += "<domain:status s='" + (*it_rm_st).s + "'/>";
    }
    to_parse["rem"] += "</domain:rem>";
  }
  else
    to_parse["rem"] = "";

  //registrant of tag $(chg)$ and
  string registrant_str = _command->get_registrant();
  if (registrant_str != "")
    registrant_str = "<domain:registrant>" + registrant_str
      + "</domain:registrant>";

  //auth info of tag $(chg)$
  AuthInfo authInfo;
  authInfo = _command->get_authInfo();
  string auth_info_str;
  auth_info_str = "<domain:authInfo>";
  
  if (authInfo.get_roid() == "") {
    auth_info_str += "<domain:pw>" + authInfo.get_pw() + "</domain:pw>";
  } else {
    auth_info_str += "<domain:pw roid='" + authInfo.get_roid() + "'>" + 
      authInfo.get_pw() + "</domain:pw>";
  }
  auth_info_str += "</domain:authInfo>";
  to_parse["chg"] = "<domain:chg>" + registrant_str + auth_info_str + 
    "</domain:chg>";
  

  //clTRID
  to_parse["clTRID"] = clTRID;

  _xml = str_util.parse(xml_template, to_parse, "$(", ")$");
}

LIBEPP_NICBR_NS_END
