/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: DomainInfoTest.cpp 407 2006-02-10 18:28:36Z eduardo $ */
#include <memory>

#include "libepp_nicbr.H"

#include "DomainInfoTest.H"
#include "DomainInfo.H"
#include "FileUtil.H"
#include "IoException.H"
#include "XmlException.H"

using std::auto_ptr;

#include <iostream>
using namespace std;

LIBEPP_NICBR_NS_USE

CPPUNIT_TEST_SUITE_REGISTRATION(DomainInfoTest);

DomainInfoTest::DomainInfoTest() {}

DomainInfoTest::~DomainInfoTest() {}

void DomainInfoTest::setUp() {}

void DomainInfoTest::tearDown() {}

void DomainInfoTest::set_xml_template_test()
{
  string to_be_parsed =
    "<command>"
      "<info>"
        "<domain:info "
          "xmlns:domain='urn:ietf:params:xml:ns:domain-1.0' "
          "xsi:schemaLocation='urn:ietf:params:xml:ns:domain-1.0 "
          "domain-1.0.xsd'>"
          "<domain:name hosts='$(hosts_control)$'>"
          "EmmaWatsonIsCharming.com</domain:name>"
          "$(auth_info)$"
        "</domain:info>"
      "</info>"
      "<clTRID>$(clTRID)$</clTRID>"
    "</command>";

  DomainInfo domain_info;
  DomainInfoCmd* domain_info_cmd = domain_info.get_command();
  domain_info_cmd->set_name("EmmaWatsonIsCharming.com");
  domain_info_cmd->set_hosts_control("sub");

  AuthInfo auth;
  auth.set_roid("SH8013-REP");
  auth.set_pw("abc123");
  domain_info_cmd->set_authInfo(auth);

  CPPUNIT_ASSERT_NO_THROW(domain_info.set_xml_template(to_be_parsed,
						      "ABC-12345"));

  string expected =
    "<command>"
      "<info>"
        "<domain:info "
          "xmlns:domain='urn:ietf:params:xml:ns:domain-1.0' "
          "xsi:schemaLocation='urn:ietf:params:xml:ns:domain-1.0 "
          "domain-1.0.xsd'>"
          "<domain:name hosts='sub'>EmmaWatsonIsCharming.com</domain:name>"
          "<domain:authInfo>"
            "<domain:pw roid='SH8013-REP'>abc123</domain:pw>"
          "</domain:authInfo>"
        "</domain:info>"
        "</info>"
      "<clTRID>ABC-12345</clTRID>"
    "</command>";

  CPPUNIT_ASSERT_EQUAL(expected, domain_info.get_xml_command());

  // tests reset method
  domain_info_cmd = domain_info.get_command();
  domain_info_cmd->reset();

  CPPUNIT_ASSERT(domain_info_cmd->get_name() == "");
  CPPUNIT_ASSERT(domain_info_cmd->get_hosts_control() == "all");

  CPPUNIT_ASSERT(domain_info_cmd->get_authInfo().get_pw() == "");
  CPPUNIT_ASSERT(domain_info_cmd->get_authInfo().get_roid() == "");
}

void DomainInfoTest::command_test()
{
  DomainInfo domain_info;
  DomainInfoCmd *domain_info_cmd = domain_info.get_command();

  domain_info_cmd->set_name("EmmaWatsonIsCharming.com");
  domain_info_cmd->set_hosts_control("sub");

  AuthInfo auth;
  auth.set_roid("SH8013-REP");
  auth.set_pw("abc123");
  domain_info_cmd->set_authInfo(auth);

  bool exception_caught = false;
  try {
    string xml_template = 
      FileUtil::read_file("../docs/templates/domain_info.xml");
    domain_info.set_xml_template(xml_template, "ABC-12345");

    DomParser parser;
    parser.enable_validation("../docs/schemas");
    parser.parse_command(domain_info.get_xml_command());
  } catch (const IoException &e) {
    exception_caught = true;
    printf("\nIO Exception: code [%d] message [%s]",
	e.get_code(), e.get_msg().c_str());
  } catch (const XmlException &e) {
    exception_caught = true;
    printf("\nXml Exception: code [%d] message [%s] low level message [%s]\n",
	   e.get_code(), e.get_msg().c_str(), e.get_low_level_msg().c_str());
  }

  CPPUNIT_ASSERT(!exception_caught);
}

void DomainInfoTest::response_test()
{
  string expected =
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>"
      "<epp xmlns=\"urn:ietf:params:xml:ns:epp-1.0\" "
           "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
           "xsi:schemaLocation=\"urn:ietf:params:xml:ns:epp-1.0 "
           "epp-1.0.xsd\">"
        "<response>"
           "<result code=\"1000\">"
              "<msg>Command completed successfully</msg>"
           "</result>"
           "<resData>"
             "<domain:infData "
              "xmlns:domain=\"urn:ietf:params:xml:ns:domain-1.0\" "
              "xsi:schemaLocation=\"urn:ietf:params:xml:ns:domain-1.0 "
              "domain-1.0.xsd\">"
               "<domain:name>EmmaWatsonIsCharming.org</domain:name>"
               "<domain:roid>YADAYADAYADA-ABC</domain:roid>"
               "<domain:status s=\"clientHold\"/>"
               "<domain:status s=\"serverUpdateProhibited\"/>"
               "<domain:registrant>SH8013-REP</domain:registrant>"
               "<domain:contact type=\"admin\">Voldemort</domain:contact>"
               "<domain:contact type=\"tech\">Dumbledore</domain:contact>"
               "<domain:ns>"
                 "<domain:hostAttr>"
                   "<domain:hostName>ns1.EmmaWatsonIsCharming.org"
                   "</domain:hostName>"
                   "<domain:hostAddr>192.168.0.1</domain:hostAddr>"
                 "</domain:hostAttr>"
                 "<domain:hostAttr>"
                   "<domain:hostName>ns1.granger.com</domain:hostName>"
                 "</domain:hostAttr>"
               "</domain:ns>"
               "<domain:clID>MrPotter</domain:clID>"
               "<domain:crID>Weasley</domain:crID>"
               "<domain:crDate>1999-04-03T22:00:00.0Z</domain:crDate>"
               "<domain:upID>Sirius</domain:upID>"
               "<domain:upDate>2002-12-12T22:00:00.0Z</domain:upDate>"
               "<domain:exDate>2001-04-03T22:00:00.0Z</domain:exDate>"
               "<domain:trDate>2005-01-03T22:00:00.0Z</domain:trDate>"
               "<domain:authInfo>"
               "<domain:pw>FooBar321</domain:pw>"
               "</domain:authInfo>"
             "</domain:infData>"
           "</resData>"
           "<trID>"
             "<clTRID>ABC-12345</clTRID>"
             "<svTRID>54321-XYZ</svTRID>"
           "</trID>"
        "</response>"
      "</epp>";

  DomParser parser;
  parser.enable_validation("../docs/schemas");

  DomainInfo domain_info;
  CPPUNIT_ASSERT_NO_THROW(domain_info.set_response(expected, &parser));

  DomainInfoRsp* domain_info_rsp = domain_info.get_response();

  CPPUNIT_ASSERT(domain_info_rsp != 0);
  string result_code = "1000";
  CPPUNIT_ASSERT_EQUAL(Response::OK, domain_info_rsp->get_result_code());

  string response =
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>"
    "<epp xmlns=\"urn:ietf:params:xml:ns:epp-1.0\" "
    "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
    "xsi:schemaLocation=\"urn:ietf:params:xml:ns:epp-1.0 "
    "epp-1.0.xsd\">"
    "<response>"
    "<result code=\"" + result_code + "\">"
    "<msg>" + domain_info_rsp->get_result_msg() + "</msg>"
    "</result>"
    "<resData>"
    "<domain:infData "
    "xmlns:domain=\"urn:ietf:params:xml:ns:domain-1.0\" "
    "xsi:schemaLocation=\"urn:ietf:params:xml:ns:domain-1.0 "
    "domain-1.0.xsd\">";

  //get_name
  //get_roid
  response +=
    "<domain:name>" + domain_info_rsp->get_name() + "</domain:name>"
    "<domain:roid>" + domain_info_rsp->get_roid() + "</domain:roid>";

  //get_status_set
  set<string> status_set = domain_info_rsp->get_status_set();
  set<string>::const_iterator it;
  for (it = status_set.begin(); it != status_set.end(); it++) {
    response += "<domain:status s=\"" + (*it) + "\"/>";
  }

  //get_registrant
  response +=
    "<domain:registrant>" + domain_info_rsp->get_registrant() +
    "</domain:registrant>";

  //get_contacts
  map< string, string, less<string> > contacts = domain_info_rsp->get_contacts();
  map< string, string, less<string> >::const_iterator it_map;
  for (it_map = contacts.begin(); it_map != contacts.end(); it_map++) {
    response += "<domain:contact type=\"" + (*it_map).first + "\">" +
      (*it_map).second + "</domain:contact>";
  }

  //get_nameservers
  response += "<domain:ns>";
  set<struct NameServer> nameservers = domain_info_rsp->get_nameservers();
  set<struct NameServer>::const_iterator it_ns;
  for (it_ns = nameservers.begin(); it_ns != nameservers.end(); it_ns++) {
    response += "<domain:hostAttr>" ;
    response += "<domain:hostName>" + it_ns->name + "</domain:hostName>";
    for (it = it_ns->ips.begin(); it != it_ns->ips.end(); it++) {
      response += "<domain:hostAddr>" + (*it) + "</domain:hostAddr>";
    }
    response += "</domain:hostAttr>";
  }
  response += "</domain:ns>";

  //get_clID
  response += "<domain:clID>" + domain_info_rsp->get_clID() +
    "</domain:clID>";

  //get_crID
  //get_crDate
  response += "<domain:crID>" + domain_info_rsp->get_crID() + "</domain:crID>"
    "<domain:crDate>" + domain_info_rsp->get_crDate() + "</domain:crDate>";

  //get_upID
  response += "<domain:upID>" + domain_info_rsp->get_upID() + "</domain:upID>";

  //get_upDate
  //get_exDate
  //get_trDate
  response +=
    "<domain:upDate>" + domain_info_rsp->get_upDate() + "</domain:upDate>"
    "<domain:exDate>" + domain_info_rsp->get_exDate() + "</domain:exDate>"
    "<domain:trDate>" + domain_info_rsp->get_trDate() + "</domain:trDate>";

  //get_pw
  response +=
    "<domain:authInfo>"
    "<domain:pw>" + domain_info_rsp->get_authInfo().get_pw() + "</domain:pw>"
    "</domain:authInfo>";

  response +=
    "</domain:infData>"
    "</resData>"
    "<trID>"
    "<clTRID>" + domain_info_rsp->get_clTRID() + "</clTRID>"
    "<svTRID>" + domain_info_rsp->get_svTRID() + "</svTRID>"
    "</trID>"
    "</response>"
    "</epp>";

  CPPUNIT_ASSERT_EQUAL(expected, response);

  // tests reset method
  domain_info_rsp->reset();
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_name());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_roid());
  CPPUNIT_ASSERT(domain_info_rsp->get_status_set().empty());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_registrant());
  CPPUNIT_ASSERT(domain_info_rsp->get_contacts().empty());
  CPPUNIT_ASSERT(domain_info_rsp->get_nameservers().empty());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_clID());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_crID());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_crDate());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_upID());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_exDate());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_upDate());
  CPPUNIT_ASSERT_EQUAL((const string)"", domain_info_rsp->get_trDate());
  CPPUNIT_ASSERT_EQUAL((const string)"",
		       domain_info_rsp->get_authInfo().get_roid());
  CPPUNIT_ASSERT_EQUAL((const string)"",
		       domain_info_rsp->get_authInfo().get_pw());
}
