/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: DomainUpdateTest.cpp 398 2006-02-10 16:39:40Z cleber $ */
#include <memory>

#include "libepp_nicbr.H"

#include "DomainUpdateTest.H"
#include "DomainUpdate.H"
#include "FileUtil.H"
#include "IoException.H"
#include "XmlException.H"

using std::auto_ptr;

LIBEPP_NICBR_NS_USE

CPPUNIT_TEST_SUITE_REGISTRATION(DomainUpdateTest);

DomainUpdateTest::DomainUpdateTest() {}

DomainUpdateTest::~DomainUpdateTest() {}

void DomainUpdateTest::setUp() {}

void DomainUpdateTest::tearDown() {}

void DomainUpdateTest::set_xml_template_test()
{
  string to_be_parsed =
    "<command>"
    "<update>"
    "<domain:update "
    "xmlns:domain='urn:ietf:params:xml:ns:domain-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:domain-1.0 "
    "domain-1.0.xsd'>"
    "<domain:name>$(name)$</domain:name>"
    "$(add)$"
    "$(rem)$"
    "$(chg)$"
    "</domain:update>"
    "</update>"
    "<clTRID>$(clTRID)$</clTRID>"
    "</command>";

  DomainUpdate domain_update;
  DomainUpdateCmd* domain_update_cmd = domain_update.get_command();
  domain_update_cmd->set_name("example.org");
  
  struct NameServer ns;
  ns.name = "ns0.example.org";
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns1.example.org";
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns3.example.org";
  ns.ips.insert("192.168.0.1");
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns2.example.org";
  domain_update_cmd->insert_nameserver_rem(ns);
  domain_update_cmd->insert_contact_add("tech", "XXX");
  domain_update_cmd->insert_contact_rem("admin", "ABC");

  struct DomainUpdateCmd::Status st;
  st.s = "clientHold";
  st.lang = "en";
  st.msg = "Payment overdue.";
  domain_update_cmd->insert_status_add(st);

  st.s = "clientUpdateProhibited";
  st.lang = "";
  st.msg = "";
  domain_update_cmd->insert_status_rem(st);

  domain_update_cmd->set_registrant("registro.br");

  AuthInfo authInfo;
  authInfo.set_roid("");
  authInfo.set_pw("abc123");
  domain_update_cmd->set_authInfo(authInfo);

  CPPUNIT_ASSERT_NO_THROW(domain_update.set_xml_template(to_be_parsed,
							"ABC-12345"));

  string expected =
    "<command>"
    "<update>"
    "<domain:update "
    "xmlns:domain='urn:ietf:params:xml:ns:domain-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:domain-1.0 "
    "domain-1.0.xsd'>"
    "<domain:name>example.org</domain:name>"
    "<domain:add>"
    "<domain:ns>"
    "<domain:hostAttr>"
    "<domain:hostName>ns0.example.org</domain:hostName>"
    "</domain:hostAttr>"
    "<domain:hostAttr>"
    "<domain:hostName>ns1.example.org</domain:hostName>"
    "</domain:hostAttr>"
    "<domain:hostAttr>"
    "<domain:hostName>ns3.example.org</domain:hostName>"
    "<domain:hostAddr>192.168.0.1</domain:hostAddr>"
    "</domain:hostAttr>"
    "</domain:ns>"
    "<domain:contact type='tech'>XXX</domain:contact>"
    "<domain:status s='clientHold' lang='en'>Payment overdue.</domain:status>"
    "</domain:add>"
    "<domain:rem>"
    "<domain:ns>"
    "<domain:hostAttr>"
    "<domain:hostName>ns2.example.org</domain:hostName>"
    "</domain:hostAttr>"
    "</domain:ns>"
    "<domain:contact type='admin'>ABC</domain:contact>"
    "<domain:status s='clientUpdateProhibited'/>"
    "</domain:rem>"
    "<domain:chg>"
    "<domain:registrant>"
    "registro.br"
    "</domain:registrant>"
    "<domain:authInfo>"
    "<domain:pw>abc123</domain:pw>"
    "</domain:authInfo>"
    "</domain:chg>"
    "</domain:update>"
    "</update>"
    "<clTRID>ABC-12345</clTRID>"
    "</command>";

  CPPUNIT_ASSERT_EQUAL(expected, domain_update.get_xml_command());

  // tests reset method
  domain_update_cmd = domain_update.get_command();
  domain_update_cmd->reset();
  CPPUNIT_ASSERT(domain_update_cmd->get_name() == "");
  CPPUNIT_ASSERT(domain_update_cmd->get_nameserver_add().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_nameserver_rem().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_contact_add().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_contact_rem().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_status_add().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_status_rem().empty());
  CPPUNIT_ASSERT(domain_update_cmd->get_registrant() == "");
  CPPUNIT_ASSERT(domain_update_cmd->get_authInfo().get_pw() == "");
}

void DomainUpdateTest::command_test()
{
  DomainUpdate domain_update;
  DomainUpdateCmd *domain_update_cmd = domain_update.get_command();

  domain_update_cmd->set_name("example.org");
  
  struct NameServer ns;
  ns.name = "ns0.example.org";
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns1.example.org";
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns3.example.org";
  ns.ips.insert("192.168.0.1");
  domain_update_cmd->insert_nameserver_add(ns);
  ns.name = "ns2.example.org";
  domain_update_cmd->insert_nameserver_rem(ns);
  domain_update_cmd->insert_contact_add("tech", "XXX");
  domain_update_cmd->insert_contact_rem("admin", "ABC");

  struct DomainUpdateCmd::Status st;
  st.s = "clientHold";
  st.lang = "en";
  st.msg = "Payment overdue.";
  domain_update_cmd->insert_status_add(st);

  st.s = "clientUpdateProhibited";
  st.lang = "";
  st.msg = "";
  domain_update_cmd->insert_status_rem(st);

  domain_update_cmd->set_registrant("registro.br");

  AuthInfo authInfo;
  authInfo.set_roid("");
  authInfo.set_pw("abc123");
  domain_update_cmd->set_authInfo(authInfo);

  bool exception_caught = false;
  try {
    string xml_template = 
      FileUtil::read_file("../docs/templates/domain_update.xml");
    domain_update.set_xml_template(xml_template, "ABC-12345");

    DomParser parser;
    parser.enable_validation("../docs/schemas");
    parser.parse_command(domain_update.get_xml_command());
  } catch (const IoException &e) {
    exception_caught = true;
    printf("\nIO Exception: code [%d] message [%s]",
	e.get_code(), e.get_msg().c_str());
  } catch (const XmlException &e) {
    exception_caught = true;
    printf("\nXml Exception: code [%d] message [%s] low level message [%s]\n",
	   e.get_code(), e.get_msg().c_str(), e.get_low_level_msg().c_str());
  }

  CPPUNIT_ASSERT(!exception_caught);
}

void DomainUpdateTest::response_test()
{
  string expected =
    "<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
    "<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
    "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
    "epp-1.0.xsd'>"
    "<response>"
    "<result code='1000'>"
    "<msg>Command completed successfully</msg>"
    "</result>"
    "<trID>"
    "<clTRID>ABC-12345</clTRID>"
    "<svTRID>54321-XYZ</svTRID>"
    "</trID>"
    "</response>"
    "</epp>";

  DomParser parser;
  DomainUpdate domain_update;
  Response* domain_update_rsp = domain_update.get_response();

  CPPUNIT_ASSERT_NO_THROW(domain_update.set_response(expected, &parser));
  domain_update_rsp = domain_update.get_response();

  string result_code = "1000";
  CPPUNIT_ASSERT_EQUAL(domain_update_rsp->get_result_code(), Response::OK);

  string response = 
    "<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
    "<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
    "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
    "epp-1.0.xsd'>"
    "<response>"
    "<result code='" + result_code + "'>"
    "<msg>" + domain_update_rsp->get_result_msg() + "</msg>"
    "</result>"
    "<trID>"
    "<clTRID>" + domain_update_rsp->get_clTRID() + "</clTRID>"
    "<svTRID>" + domain_update_rsp->get_svTRID() + "</svTRID>"
    "</trID>"
    "</response>"
    "</epp>";

  CPPUNIT_ASSERT_EQUAL(expected, response);
}
